module Umbreo
	module Models
		class Authentication

			# 
			# Init data
			# 
			def initialize(email = nil, password = nil)
				config    = Configuration.new

				@email    = email
				@password = password
				@endpoint = (config.read_file_authentication[:end_point] rescue nil) || SERVER_END_POINT
 				@message  = Helpers::AlertMessage
				@errors   = []
			end

			# 
			# Check credentails validation
			# 
			def valid?
				@errors << "Email or Password Can not be blank!" if @email.blank? || @password.blank?
				if @email.present?
					@errors << "Invalid format email" if @email.match(/\A([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})\z/i).blank?
				end

				@errors.blank?
			end

			# 
			# return error from validation
			# 
			def error
				@errors.first
			end

			# 
			# Access server and get access token
			# 
			def sign_in_and_retrieve_token!
				begin
					_request = 	Typhoeus.post(
									      "#{@endpoint}/api/v1/users/retrieve_token",
									      body: {
									        email: @email,
									        password: @password,
									      }
									    )

		    	@data = JSON.parse _request.response_body
				rescue Exception => e
					@data = { message: "An error of type #{e.class} happened, #{e.message}"}
				end
			end

			# 
			# remove file configuration
			# 
			def sign_out
				config = Configuration.new
				config.delete
				puts 'Local credentials cleared.'
			end

			# 
			# Action after retrive token
			# 
			def sign_in_callback
				if @data['success']
					@message.show_success_message("Setting up configuration file at ~/.umbreoconfig...")

					config        = Configuration.new
					end_point_url = config.read_file_authentication[:end_point] rescue nil
					config.write_file_authentication(@data, end_point_url)
					
					sleep 1
					puts "Logged in as #{ Rainbow(@data['email']).cyan }"
				else
					@message.show_error_message(@data['message'])
				end
			end

			# 
			# check current_user
			# 
			def retrieve_credentails!
				config = Configuration.new

				if config.is_valid_user?
					config.read_file_authentication
				else
					@message.show_error_message("Invalid credentials. Please try to login `umbreo login` again.")
					exit!(0)
				end
			end

		end
	end
end