module Umbreo
	module Models
		class CloudProvider

			# 
			# available provider with slug name
			# 
			CLOUDS = [:aws, :gcompute, :docean, :openstack, :XenServer]
			
			# 
			# Init Global Variable
			# 
			def initialize(credentials = {}, slug_or_id = nil)
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end

				@errors = []
				@slug_or_id = slug_or_id
			end

			# 
			# get all available data cloud provider 
			# 
			def retrieve_data
				Helpers::ErrorException.rescue do
					data = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/cloud_providers",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email
					          }
					        )

		    	@data = JSON.parse data.response_body
				end
			end

			# 
			# callback for retrieve data
			# 
			def all
				retrieve_data

	    	if @data['umbreo_cloud_providers'].present?
          Helpers::Table.show_table(@data['umbreo_cloud_providers'], "List Cloud Provider", ['ID', 'Name', 'Slug', 'Description'])
	    	else
	    		Helpers::AlertMessage.show_error_message(@data["message"])
	    	end
			end

			# 
			# export params json of cloud provider
			# 
			def export
				@errors << "ID or Slug is required." if @slug_or_id.blank?
				@errors << "ID or Slug is not found." if CLOUDS.exclude? @slug_or_id.to_sym

				if valid?
					Helpers::ErrorException.rescue do
						response = Typhoeus.get(
	            "#{@endpoint}/api/v1/cloud_providers/export",
	            body: {
	              authenticate_token:   @api_key,
	              email:                @email,
	              id:                   @slug_or_id
	            }
	          ) 
	          
		        @response = JSON.parse response.response_body rescue nil
		        if @response["umbreo_cloud_provider"].present?
		        	name_file = @response["umbreo_cloud_provider"]["name"].downcase.titleize.delete(" ").delete("/").underscore
		        	Helpers::FileGenerator.create(name_file, @response["umbreo_cloud_provider"])
	            Helpers::AlertMessage.show_success_message("Success export cloud provider. your cloud_provider is saved on json file with name #{name_file}.json")
		        else
		        	Helpers::AlertMessage.show_error_message(@response["message"])
		        end
	      	end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# check valid json params
			# 
			def validate(file_path)
				@errors << "File directory is required." if file_path.blank?

				begin
					file    = File.open(file_path, "r")
					content = file.read
	        json    = JSON.parse(content)
	        encode  = Helpers::JsonBaseConvert.encode(json)
				rescue
					@errors << "Error on file. Please check again."
				end
				@errors << "Content of file is required." if content.blank?

				if valid?
					Helpers::ErrorException.rescue do

	          response = Typhoeus.get(
	            "#{@endpoint}/api/v1/cloud_providers/validate",
	            body: {
	              authenticate_token: @api_key,
	              email:              @email,
	              content:            encode
	            }
	          )

	          @response = JSON.parse response.response_body

	          if @response["success"]
							Helpers::AlertMessage.show_success_message(@response["message"])
	          else
							Helpers::AlertMessage.show_error_message(@response["message"])
	          end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# check if no errors
			# 
			def valid?
				@errors.blank?
			end

			# 
			# get first of errors
			# 
			def error
				@errors.first
			end

		end
	end
end