module Umbreo
	module Models
		class GoogleComputeEngineProvider

			# 
			# init params which required for create cloud provider
			# 
			def initialize(credentials = {}, name_or_id = nil)
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end

				@errors     = []
				@name_or_id = name_or_id
			end

			
			# 
			# get list size of amazon web service
			# 
			def region_list
				@errors << "ID or Name provider is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/google_compute_engine_services/region_list",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            id:                 @name_or_id
						          } 
						        )

			    	data = JSON.parse data.response_body

			    	if data['success']
							Helpers::Table.table_callback_retrieve_data(data['regions'], "List GCE Region")
			    	else
			    		Helpers::AlertMessage.show_error_message(data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end


			# 
			# get list machine of google
			# 
			def machine_list
				@errors << "ID or Name provider is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/google_compute_engine_services/machine_list",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            id:                 @name_or_id
						          } 
						        )

			    	data = JSON.parse data.response_body

			    	if data['success']
	          	Helpers::Table.show_table(data['machines'], "List GCE Machine", ['Machine Description', 'Machine Name'])
			    	else
			    		Helpers::AlertMessage.show_error_message(data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# get list machine of google
			# 
			def image_list
				@errors << "ID or Name provider is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/google_compute_engine_services/image_list",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            id:                 @name_or_id
						          } 
						        )

			    	data = JSON.parse data.response_body

			    	if data['success']
	          	Helpers::Table.show_table(data['images'], "List GCE Image", ['Image ID', 'Image Name'])
			    	else
			    		Helpers::AlertMessage.show_error_message(data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# get list network of google
			# 
			def network_list
				@errors << "ID or Name provider is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/google_compute_engine_services/network_list",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            id:                 @name_or_id
						          } 
						        )

			    	data = JSON.parse data.response_body

			    	if data['success']
							Helpers::Table.table_callback_retrieve_data(data['networks'], "List GCE Network")
			    	else
			    		Helpers::AlertMessage.show_error_message(data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# get json credential file
			# 
			def export
				@errors << "ID or Slug is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = Typhoeus.get(
		            "#{@endpoint}/api/v1/google_compute_engine_services/export",
		            body: {
		              authenticate_token:   @api_key,
		              email:                @email,
		              id:                   @name_or_id
		            }
		          )

		        data = JSON.parse data.response_body rescue nil
		        user_provider = data["user_provider"]

						if user_provider.present?
		        	name_file = "google_compute_engine_service_provider"
		        	Helpers::FileGenerator.create(name_file, user_provider)
		          Helpers::AlertMessage.show_success_message("Success export google compute engine provider. Your file is saved on json file with name #{name_file}.json")
		        else
		        	Helpers::AlertMessage.show_error_message(data["message"])
		        end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# check valid json params
			# 
			def validate(file_path)
				@errors << "File directory is required." if file_path.blank?

				begin
					file    = File.open(file_path, "r")
					content = file.read
	        json    = JSON.parse(content)
	        encode  = Helpers::JsonBaseConvert.encode(json)
				rescue
					@errors << "Error on file. Please check again."
				end

				@errors << "Content of file is required." if content.blank?

				if valid?
					Helpers::ErrorException.rescue do

	          response = Typhoeus.get(
	            "#{@endpoint}/api/v1/google_compute_engine_services/validate",
	            body: {
	              authenticate_token: @api_key,
	              email:              @email,
	              content:            encode
	            }
	          )

	          @response = JSON.parse response.response_body

	          if @response["success"]
							Helpers::AlertMessage.show_success_message(@response["message"])
	          else
							Helpers::AlertMessage.show_error_message(@response["message"])
	          end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# check no errors
			# 
			def valid?
				@errors.blank?
			end

			# 
			# call errors
			# 
			def error
				@errors.first
			end

		end	
	end
end