module Umbreo
	module Models
		class Instance

			# 
			# list deployment type of instance
			# 
			DEPLOYMENT_TYPE = ["test", "manual", "provider"]
			
			def initialize(credentials = {}, name_or_id = nil)
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end

				@name_or_id = name_or_id
				@errors     = []
			end

			# 
			# retrieve data instance
			# 
			def retrieve_data(filter = {})
				Helpers::ErrorException.rescue do
					data = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/instances",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email,
					            filter:             filter
					          }
					        )
						    
		    	@data = JSON.parse data.response_body
				end
			end

			# 
			# callback for retrieve data
			# 
			def all(filter = {})
				@errors << "Please choose correct one of regiter type |all, registered, unregistered|" if ["registered", "unregistered"].exclude?(filter[:register_type]) && filter[:register_type].present?
				@errors << "Please choose correct one of type deployment |all, #{DEPLOYMENT_TYPE.join(", ")}|" if DEPLOYMENT_TYPE.exclude?(filter[:type]) && filter[:type].present?
				@errors << "Please choose correct one of cloud provider |all, #{CloudProvider::CLOUDS.join(", ")}|" if CloudProvider::CLOUDS.exclude?(filter[:provider]) && filter[:provider].present?
				@errors << "Search keyword is required." if filter[:keyword].blank? && filter[:search]

				if valid?
					retrieve_data(filter)

		    	if @data['user_nodes'].present?
	          Helpers::Table.show_table(@data['user_nodes'], "List My Instance#{ (' | Page: ' + filter[:page].to_s) if @data['total_page'] > 1 }", ['ID', 'Name', 'Hostname', 'Description'])
		    	else
		    		Helpers::AlertMessage.show_error_message(@data["message"])
		    	end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# get instance data
			# 
			def show
				@errors << "ID or Name is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/instances/show",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            name_or_id:         @name_or_id
						          }
						        )

			    	@data = JSON.parse data.response_body

			    	if @data['success']
			    		type = @data['type_deployment'] == 'barebone' ? 'manual' : @data['type_deployment']
	          	Helpers::AlertMessage.show_content_information_message(@data['id']) { "ID: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['name']) { "Name: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['description']) { "Description: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['role']) { "Blueprint: #message" }
	          	Helpers::AlertMessage.show_content_information_message(type) { "Deployment type: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['hostname']) { "Hostname: #message" }
	          	Helpers::AlertMessage.show_content_information_message(Date.parse(@data['created_at']).to_s) { "Created At: #message" }
			    	else
			    		Helpers::AlertMessage.show_error_message(@data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# deploy blueprint
			# 
			def deploy(parameters = {})
				
				@error << "Name is required" if parameters[:name].blank?
				@error << "Blueprint Json file is required" if parameters[:blueprint].blank?
				@error << "Deployment type is required" if parameters[:type].blank?
				@error << "Compute Provider Json file is required" if parameters[:provider].blank? && parameters[:type] == "provider"

				file_parameter = {}

				[:blueprint, :service_logging, :service_monitoring, :service_backup, :provider].each do |file_name|
					file_path = parameters[file_name]

					if file_path.present?
						begin
							file    = File.open(file_path, "r")
							content = file.read
			        json    = JSON.parse(content)
			        file_parameter[file_name] = Helpers::JsonBaseConvert.encode(json)
						rescue
							@errors << "Error on #{ file_path } file. Please check again."
						end
					end
				end

				if valid?
					Helpers::ErrorException.rescue do

	          response = Typhoeus.post(
	            "#{@endpoint}/api/v1/instances/deploy",
	            body: {
	              authenticate_token: @api_key,
	              email:              @email,
	              name:               parameters[:name],
	              description:        parameters[:desc],
	              type:               parameters[:type],
	              custom:             parameters[:custom],
	              file_parameter:     file_parameter
	            }
	          )

	          @response = JSON.parse response.response_body

	          if @response["success"]
							Helpers::AlertMessage.show_success_message(@response["message"])
	          else
							Helpers::AlertMessage.show_error_message(@response["message"])
	          end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# Check no error from validation
			# 
			def valid?
				@errors.blank?
			end

			# 
			# Get first error
			# 
			def error
				@errors.first
			end

		end
	end
end