module Umbreo
	module Models
		class Profile
			
			def initialize(credentials = {})
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end
			end

			def retrieve_data(filter = {})
				Helpers::ErrorException.rescue do
					data = Typhoeus.get(
						          "#{@endpoint}/api/v1/profiles",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            filter:             filter
						          }
						        )
						        
		    	@data = JSON.parse data.response_body
				end
			end

			def all(filter = {})
				@errors << "Search keyword is required." if filter[:keyword].blank? && filter[:search]
				
				if valid?
					retrieve_data(filter)

		    	if @data['umbreo_profiles'].present?
		    		page = " | Page: #{filter[:page]} of #{@data['total_page']} pages"
	          Helpers::Table.show_table(@data['umbreo_profiles'], "List Profile#{ page if @data['total_page'] > 1 }", ['ID', 'Name', 'Description'])
		    	else
		    		Helpers::AlertMessage.show_error_message(@data["message"])
		    	end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# Check no error from validation
			# 
			def valid?
				@errors.blank?
			end

			# 
			# Get first error
			# 
			def error
				@errors.first
			end

		end
	end
end