module Umbreo
	module Models
		class Provider

			# 
			# init params which required for create cloud provider
			# 
			def initialize(credentials = {}, name_or_id = nil)
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end

				@errors     = []
				@name_or_id = name_or_id
			end

			# 
			# get list of my providers
			# 
			def retrieve_data(filter = {})
				Helpers::ErrorException.rescue do
					data = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/providers",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email,
					            filter:             filter
					          }
					        )
						    
		    	@data = JSON.parse data.response_body
				end
			end

			# 
			# callback for retrieve data
			# 
			def all(filter = {})
				@errors << "Please choose correct option cloud provider |all, aws, gcompute, docean, openstack, XenServer|" if CloudProvider::CLOUDS.exclude?(filter[:umbreo_cloud_provider_id].try(:to_sym)) && filter[:umbreo_cloud_provider_id].present?

				if valid?
					retrieve_data(filter)

		    	if @data['user_providers'].present?
						page = " | Page: #{filter[:page]} of #{@data['total_page']} pages"
	          Helpers::Table.show_table(@data['user_providers'], "List My Provider#{ page if @data['total_page'] > 1 }", ['ID', 'Name', 'Cloud Provider', 'Description'])
		    	else
		    		Helpers::AlertMessage.show_error_message(@data["message"])
		    	end
				else
	    		Helpers::AlertMessage.show_error_message(error)
				end
				
			end

			# 
			# get instance data
			# 
			def show
				@errors << "ID or Name is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/providers/show",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            name_or_id:         @name_or_id
						          }
						        )

			    	@data = JSON.parse data.response_body

			    	if @data['success']
	          	Helpers::AlertMessage.show_content_information_message(@data['id']) { "ID: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['name']) { "Name: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['description']) { "Description: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['umbreo_cloud_provider']) { "Cloud provider: #message" }
	          	Helpers::AlertMessage.show_content_information_message(Date.parse(@data['created_at']).to_s) { "Created At: #message" }
			    	else
			    		Helpers::AlertMessage.show_error_message(@data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# validation params
			# 
			def check_validation!
				@errors << 'Invalid cloud provider' if CloudProvider::CLOUDS.exclude?(@cloud_provider.to_sym)
				@errors << 'Invalid cloud provider credential' if @provider_credential.blank?
			end

			# 
			# check no errors
			# 
			def valid?
				@errors.blank?
			end

			# 
			# call errors
			# 
			def error
				@errors.first
			end

			# 
			# create provider
			# 
			def create(cloud, name, description = nil, file_path)
				@name           = name
				@desc           = description
				@cloud_provider = cloud

				@errors << "File directory is required." if file_path.blank?
				@errors << "Name is required." if name.blank?
				begin
					file    = File.open(file_path, "r")
					content = file.read
	        @json   = JSON.parse(content)
	        @encode = Helpers::JsonBaseConvert.encode(@json)
				rescue
					@errors << "Error on file. Please check again."
				end
				@errors << "Content of file is required." if content.blank?

				if valid?
					url  = "#{@endpoint}/api/v1/providers"

					Helpers::ErrorException.rescue do
			 			data = Typhoeus.post(
	            url,
	            body: {
	              authenticate_token:   @api_key,
	              email:                @email,
	              provider:             @cloud_provider,
		            name_provider:        @name,
		            description_provider: @desc,
	              provider_credential:  @encode
	            }
	          )
	          data = JSON.parse data.response_body

	          if data['success']
	          	Helpers::AlertMessage.show_success_message(data['message'])
	          else
	          	Helpers::AlertMessage.show_error_message(data['message'])
	          end
	        end
				else
          Helpers::AlertMessage.show_error_message(error)
				end
			end

		end	
	end
end