module Umbreo
	module Models
		class Service
			
			def initialize(credentials = {}, name_or_id = nil)
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end

				@name_or_id = name_or_id
				@errors     = []
			end

			# 
			# retrieve data instance
			# 
			def retrieve_data(filter = {})
				Helpers::ErrorException.rescue do
					data = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/services",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email,
					            filter:             filter
					          }
					        )
						    
		    	@data = JSON.parse data.response_body
				end
			end

			# 
			# callback for retrieve data
			# 
			def all(filter = {})
				@errors << "Please choose correct state |all, active, unactive|" if ['active', 'unactive'].exclude?(filter[:state]) && filter[:state].present?
				@errors << "Search keyword is required." if filter[:keyword].blank? && filter[:search]
				
				if valid?
					retrieve_data(filter)

		    	if @data['user_services'].present?
		    		page = " | Page: #{filter[:page]} of #{@data['total_page']} pages"
	          Helpers::Table.show_table(@data['user_services'], "List Service#{ page if @data['total_page'] > 1 }", ['ID', 'Name', 'Service Provider', 'Link Type', 'Active'])
		    	else
		    		Helpers::AlertMessage.show_error_message(@data["message"])
		    	end
				else
		    	Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# get instance data
			# 
			def show
				@errors << "ID or Name is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/services/show",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            id:                 @name_or_id
						          }
						        )

			    	@data = JSON.parse data.response_body

			    	if @data['success']
	          	Helpers::AlertMessage.show_content_information_message(@data['id']) { "ID: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['name']) { "Name: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['umbreo_service']) { "Service Provider: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['link_type']) { "Link type: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['is_active']) { "Active: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['description']) { "Description: #message" }
			    	else
			    		Helpers::AlertMessage.show_error_message(@data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# export umbreo stack template params
			# 
			def export
				Helpers::ErrorException.rescue do
					response = Typhoeus.get(
	            "#{@endpoint}/api/v1/services/export",
	            body: {
	              authenticate_token:   @api_key,
	              email:                @email,
	              id:                   @name_or_id
	            }
	          ) 
	          
	        @response = JSON.parse response.response_body rescue nil
	        
	        if @response["user_service"].present?
	        	name_file = @response["user_service"]["name"].downcase.titleize.delete(" ").underscore
	        	Helpers::FileGenerator.create(name_file, @response["user_service"])

            Helpers::AlertMessage.show_success_message("Success export service. your service is saved on json file with name #{name_file}.json")
	        else
	        	Helpers::AlertMessage.show_error_message(@response["message"])
	        end
				end
			end

			# 
			# check valid json params
			# 
			def validate(file_path)
				@errors << "File directory is required." if file_path.blank?

				begin
					file    = File.open(file_path, "r")
					content = file.read
	        json    = JSON.parse(content)
	        encode  = Helpers::JsonBaseConvert.encode(json)
				rescue
					@errors << "Error on file. Please check again."
				end
				@errors << "Content of file is required." if content.blank?

				if valid?
					Helpers::ErrorException.rescue do

	          response = Typhoeus.get(
	            "#{@endpoint}/api/v1/services/validate",
	            body: {
	              authenticate_token: @api_key,
	              email:              @email,
	              content:            encode
	            }
	          )

	          @response = JSON.parse response.response_body

	          if @response["success"]
							Helpers::AlertMessage.show_success_message(@response["message"])
	          else
							Helpers::AlertMessage.show_error_message(@response["message"])
	          end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# create user provider
			# 
			def create(parameters)
				
				@errors << "Name is required" if parameters[:name].blank?

				file_path = parameters[:params]
				if file_path.present?
					begin
						file    = File.open(file_path, "r")
						content = file.read
		        json    = JSON.parse(content)
		        parameters[:params] = Helpers::JsonBaseConvert.encode(json)
					rescue
						@errors << "Error on #{ file_path } file. Please check again."
					end
				end
				
				if parameters[:user_node_attributes].present?
					main_param = parameters[:user_node_attributes]

					@errors << "#{main_param[:name]} ##{index} Blueprint Json file is required" if main_param[:name].blank?
					@errors << "#{main_param[:name]} ##{index} Deployment type is required" if main_param[:type].blank?
					@errors << "#{main_param[:name]} ##{index} Compute Provider Json file is required" if main_param[:provider].blank? && main_param[:type] == "provider"

					main_param[:file_parameter] = {}

					[:blueprint, :service_logging, :service_monitoring, :service_backup, :provider].each do |file_name|
						file_path = main_param[file_name] rescue nil

						if file_path.present?
							begin
								file    = File.open(file_path, "r")
								content = file.read
				        json    = JSON.parse(content)
				        main_param[file_name] = Helpers::JsonBaseConvert.encode(json)
							rescue
								@errors << "Error on #{ file_path } file. Please check again."
							end
						end
					end
				end

				if valid?
					Helpers::ErrorException.rescue do

	          response = Typhoeus.post(
	            "#{@endpoint}/api/v1/services",
	            body: {
	              authenticate_token: @api_key,
	              email:              @email,
	              user_service:       parameters
	            }
	          )

	          @response = JSON.parse response.response_body

	          if @response["success"]
							Helpers::AlertMessage.show_success_message("Service successfully created.")
	          else
							Helpers::AlertMessage.show_error_message(@response["message"])
	          end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end

			end

			# 
			# Check no error from validation
			# 
			def valid?
				@errors.blank?
			end

			# 
			# Get first error
			# 
			def error
				@errors.first
			end

		end
	end
end