module Umbreo
	module Models
		class ServiceProvider

			# 
			# service type
			TYPE = ["logging", "monitoring", "backup"]
			# resource type
			RESOURCE = ["master", "client"]
			
			def initialize(credentials = {}, name_or_id = nil)
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end

				@name_or_id = name_or_id
				@errors     = []
			end

			# 
			# retrieve data instance
			# 
			def retrieve_data(filter = {})
				Helpers::ErrorException.rescue do
					data = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/service_providers",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email,
					            filter:             filter
					          }
					        )
						    
		    	@data = JSON.parse data.response_body
				end
			end

			# 
			# callback for retrieve data
			# 
			def all(filter = {})
				@errors << "Please choose correct option service type |all, logging, monitoring, backup|" if TYPE.exclude?(filter[:type]) && filter[:type].present?
				@errors << "Please choose correct option resource type |all, master, client|" if RESOURCE.exclude?(filter[:resource]) && filter[:resource].present?
				@errors << "Search keyword is required." if filter[:keyword].blank? && filter[:search]
				
				if valid?
					retrieve_data(filter)

		    	if @data['umbreo_services'].present?
		    		page = " | Page: #{filter[:page]} of #{@data['total_page']} pages"
	          Helpers::Table.show_table(@data['umbreo_services'], "List Service Provider#{ page if @data['total_page'] > 1 }", ['ID', 'Name', 'Resource Type', 'Service Type', 'Slug'])
		    	else
		    		Helpers::AlertMessage.show_error_message(@data["message"])
		    	end
				else
		    	Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# get instance data
			# 
			def show(need_show = true)
				@errors << "ID or Name is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/service_providers/show",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            id:                 @name_or_id
						          }
						        )

			    	@data = JSON.parse data.response_body

			    	if @data['success']
			    		if need_show
		          	Helpers::AlertMessage.show_content_information_message(@data['id']) { "ID: #message" }
		          	Helpers::AlertMessage.show_content_information_message(@data['name']) { "Name: #message" }
		          	Helpers::AlertMessage.show_content_information_message(@data['resource_type']) { "Resource type: #message" }
		          	Helpers::AlertMessage.show_content_information_message(@data['umbreo_service_type']) { "Service Type: #message" }
		          	Helpers::AlertMessage.show_content_information_message(@data['slug']) { "Slug: #message" }
		          	Helpers::AlertMessage.show_content_information_message(@data['description']) { "Description: #message" }
			    		else
			    			@data
			    		end
			    	else
			    		Helpers::AlertMessage.show_error_message(@data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# export umbreo stack template params
			# 
			def export
				Helpers::ErrorException.rescue do
					response = Typhoeus.get(
	            "#{@endpoint}/api/v1/service_providers/export",
	            body: {
	              authenticate_token:   @api_key,
	              email:                @email,
	              id:                   @name_or_id
	            }
	          ) 
	          
	        @response = JSON.parse response.response_body rescue nil
	        
	        if @response["umbreo_service"].present?
	        	name_file = @response["umbreo_service"]["name"].downcase.titleize.delete(" ").underscore
	        	Helpers::FileGenerator.create(name_file, @response["umbreo_service"])

            Helpers::AlertMessage.show_success_message("Success export umbreo service provider. your service provider is saved on json file with name #{name_file}.json")
	        else
	        	Helpers::AlertMessage.show_error_message(@response["message"])
	        end
				end
			end

			# 
			# check valid json params
			# 
			def validate(file_path)
				@errors << "File directory is required." if file_path.blank?

				begin
					file    = File.open(file_path, "r")
					content = file.read
	        json    = JSON.parse(content)
	        encode  = Helpers::JsonBaseConvert.encode(json)
				rescue
					@errors << "Error on file. Please check again."
				end
				@errors << "Content of file is required." if content.blank?

				if valid?
					Helpers::ErrorException.rescue do

	          response = Typhoeus.get(
	            "#{@endpoint}/api/v1/service_providers/validate",
	            body: {
	              authenticate_token: @api_key,
	              email:              @email,
	              content:            encode
	            }
	          )

	          @response = JSON.parse response.response_body

	          if @response["success"]
							Helpers::AlertMessage.show_success_message(@response["message"])
	          else
							Helpers::AlertMessage.show_error_message(@response["message"])
	          end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# Check no error from validation
			# 
			def valid?
				@errors.blank?
			end

			# 
			# Get first error
			# 
			def error
				@errors.first
			end

		end
	end
end