module Umbreo
	module Models
		class Stack
			
			def initialize(credentials = {}, name_or_id = nil, status = false)
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end

				@name_or_id = name_or_id
				@is_custom  = status
				@errors     = []
			end

			def retrive_data
				Helpers::ErrorException.rescue do
					data = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/stacks",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email
					          }
					        )
						        
		    	@data = JSON.parse data.response_body
				end
			end

			def all
				retrive_data

	    	if @data['user_stacks'].present?
          Helpers::Table.show_table(@data['user_stacks'], "List My Stack", ['ID', 'Name', 'Description', 'Umbreo Stack'])
	    	else
	    		Helpers::AlertMessage.show_error_message("Stack is not found, you have to create your own stack.")
	    	end
			end

			def custom
        @custom_blueprints = @data['my'] rescue []
			end

			def show
				if @name_or_id.present?
					Helpers::ErrorException.rescue do
						stack = Typhoeus.get(
	            "#{@endpoint}/api/v1/stacks/#{@name_or_id}",
	            body: {
	              authenticate_token:   @api_key,
	              email:                @email,
	              name:                 @name_or_id,
	              status:               @is_custom
	            }
	          ) 

	          @data = JSON.parse stack.response_body rescue nil

	          if @data["success"]
	          	Helpers::AlertMessage.show_content_information_message(@data['id']) { "ID: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['name']) { "Name: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['description']) { "Description: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@data['umbreo_stack_template']) { "Stack Template: #message" }
	          	Helpers::Table.show_table(@data['user_nodes'], "List Nodes of #{@data['name']} stack", ['ID', 'Name', 'Description'])
          	else
          		Helpers::show_error_message("Stack is not found. Please try again with different name or try to search.")
          	end
					end
				else
					Helpers::AlertMessage.show_error_message("Name or id is required")
				end
			end

			# 
			# search data stack
			# 
			def search(keyword)
				Helpers::ErrorException.rescue do
					data = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/stacks/search",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email,
					            keyword:            keyword
					          }
					        )

		    	@data = JSON.parse data.response_body

		    	if @data['success']
	          Helpers::Table.show_table(@data['stacks'], "Search Result List My Stack", ['ID', 'Name', 'Description', 'Umbreo Stack'])
		    	else
		    		Helpers::AlertMessage.show_error_message(@data["message"])
		    	end
				end
			end

			# 
			# deploy stack
			# 
			def deploy(parameters = {})
				
				@errors << "Name is required" if parameters[:name].blank?
				@errors << "You need to add a instance" if parameters[:user_node_attributes].blank?

				if parameters[:user_node_attributes].present?
					parameters[:user_node_attributes].each_with_index do |user_node, index|
						main_param      = parameters[:user_node_attributes][index]

						@errors << "#{main_param[:name]} ##{index} Blueprint Json file is required" if main_param[:name].blank?
						@errors << "#{main_param[:name]} ##{index} Deployment type is required" if main_param[:type].blank?
						@errors << "#{main_param[:name]} ##{index} Compute Provider Json file is required" if main_param[:provider].blank? && main_param[:type] == "provider"

						main_param[:file_parameter] = {}

						[:blueprint, :service_logging, :service_monitoring, :service_backup, :provider].each do |file_name|
							file_path = user_node[file_name]
							if file_path.present?
								begin
									file    = File.open(file_path, "r")
									content = file.read
					        json    = JSON.parse(content)
					        main_param[file_name] = Helpers::JsonBaseConvert.encode(json)
								rescue
									@errors << "Error on #{ file_path } file. Please check again."
								end
							end
						end

					end
				end

				if valid?
					Helpers::ErrorException.rescue do

	          response = Typhoeus.post(
	            "#{@endpoint}/api/v1/stacks",
	            body: {
	              authenticate_token: @api_key,
	              email:              @email,
	              user_stack: {
		              name:                     parameters[:name],
		              description:              parameters[:desc],
		              umbreo_stack_template_id: parameters[:umbreo_stack_template_id],
		              user_nodes_attributes:    parameters[:user_node_attributes]
	              }
	            }
	          )

	          @response = JSON.parse response.response_body

	          if @response["success"]
							Helpers::AlertMessage.show_success_message("Stack successfully created.")
	          else
							Helpers::AlertMessage.show_error_message(@response["message"])
	          end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end

			end

			# 
			# Check no error from validation
			# 
			def valid?
				@errors.blank?
			end

			# 
			# Get first error
			# 
			def error
				@errors.first
			end

		end
	end
end