module Umbreo
	module Models
		class UmbreoStackTemplate
			
			def initialize(credentials = {}, name_or_id = nil)
				if credentials.present?
					@credentials = credentials
					@email       = credentials[:email]
					@api_key     = credentials[:api_key]
					@endpoint    = credentials[:end_point] || SERVER_END_POINT
				end

				@name_or_id = name_or_id
			end

			# 
			# retrieve all data umbreo stack
			# 
			def retrive_data
				Helpers::ErrorException.rescue do
					data = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/stack_templates",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email
					          }
					        )
						        
		    	@data = JSON.parse data.response_body
				end
			end

			# 
			# show umbreo stack template
			# 
			def show(just_response = false)
				Helpers::ErrorException.rescue do
					response = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/stack_templates/show",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email,
					            id:                 @name_or_id
					          }
					        )
						        
		    	@response = JSON.parse response.response_body

		    	if @response["success"]
		    		if just_response
          		@response
		    		else
	          	Helpers::AlertMessage.show_content_information_message(@response['id']) { "ID: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@response['name']) { "Name: #message" }
	          	Helpers::AlertMessage.show_content_information_message(@response['description']) { "Description: #message" }
	          	Helpers::Table.show_table(@response['roles'], "List Blueprints of #{@response['name']} stack template", ['ID Blueprint', 'Name', 'Max Server'])
		    		end

        	else
        		Helpers::show_error_message("Stack is not found. Please try again with different name or try to search.")
        	end
				end
			end

			# 
			# search data umbreo stack template
			# 
			def search(keyword)
				Helpers::ErrorException.rescue do
					data = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/stack_templates/search",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email,
					            keyword:            keyword
					          }
					        )
						        
		    	@data = JSON.parse data.response_body

		    	if @data['success']
	          Helpers::Table.show_table(@data['umbreo_stack_templates'], "List Search Result Umbreo Stack Template", ['ID', 'Name', 'Description'])
		    	else
		    		Helpers::AlertMessage.show_error_message(@data["message"])
		    	end
				end
			end

			# 
			# parsing data umbreo stack to table
			# 
			def all
				retrive_data

	    	if @data['umbreo_stack_templates'].present?
          Helpers::Table.show_table(@data['umbreo_stack_templates'], "List Umbreo Stack Template", ['ID', 'Name', 'Blueprints'])
	    	else
	    		Helpers::AlertMessage.show_error_message("Umbreo Stack Template is not found.")
	    	end
			end

			# 
			# export umbreo stack template params
			# 
			def export
				Helpers::ErrorException.rescue do
					response = Typhoeus.get(
	            "#{@endpoint}/api/v1/stack_templates/export",
	            body: {
	              authenticate_token:   @api_key,
	              email:                @email,
	              id:                   @name_or_id
	            }
	          ) 
	          
	        @response = JSON.parse response.response_body rescue nil
	        
	        if @response["umbreo_stack_template"].present?
	        	name_file = @response["umbreo_stack_template"]["name"].downcase.titleize.delete(" ").underscore
	        	Helpers::FileGenerator.create(name_file, @response["umbreo_stack_template"])

            Helpers::AlertMessage.show_success_message("Success export umbreo stack template. your stack template is saved on json file with name #{name_file}.json")
	        else
	        	Helpers::AlertMessage.show_error_message(@response["message"])
	        end
				end
			end

			# 
			# export all blueprint of umbreo stack template to files
			# 
			def export_bluprints
				Helpers::ErrorException.rescue do
					response = 	Typhoeus.get(
					          "#{@endpoint}/api/v1/stack_templates/retrieve_blueprints",
					          body: {
					            authenticate_token: @api_key,
					            email:              @email,
					            id:                 @name_or_id
					          }
					        )
						       
		    	@response = JSON.parse response.response_body

		    	if @response["data"].present?
		    		Helpers::AlertMessage.show_success_message("Creating JSON Files ...")

		    		@response["data"].each do |blueprint|
		    			new_blueprint = Models::Blueprint.new(@credentials, blueprint["id"])
		    			new_blueprint.export
		    		end

		    		Helpers::AlertMessage.show_success_message("JSON Blueprints of #{ @name_or_id } stack template already created.")
		    	else
		    		Helpers::AlertMessage.show_error_message(@response["message"])
		    	end

				end			
			end

		end
	end
end