module Umbreo
	module Models
		class XenserverProvider

			# 
			# init params which required for create cloud provider
			# 
			def initialize(credentials = {}, name_or_id = nil)
				if credentials.present?
					@email    = credentials[:email]
					@api_key  = credentials[:api_key]
					@endpoint = credentials[:end_point] || SERVER_END_POINT
				end

				@errors     = []
				@name_or_id = name_or_id
			end

			# 
			# get list storage of xenserver
			# 
			def storage_list
				@errors << "ID or Name provider is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/xenserver_services/storage_list",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            id:                 @name_or_id
						          } 
						        )

			    	data = JSON.parse data.response_body

			    	if data['success']
							Helpers::Table.table_callback_retrieve_data(data['storages'], "List Xenserver Storage")
			    	else
			    		Helpers::AlertMessage.show_error_message(data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end
			
			# 
			# get list vhd of xenserver
			# 
			def vhd_list
				@errors << "ID or Name provider is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/xenserver_services/vhd_list",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            id:                 @name_or_id
						          } 
						        )

			    	data = JSON.parse data.response_body

			    	if data['success']
	          	Helpers::Table.show_table(data['vhd_files'], "List VHD File", ['VHD ID', 'VHD File'])
			    	else
			    		Helpers::AlertMessage.show_error_message(data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# get list net of xenserver
			# 
			def net_list
				@errors << "ID or Name provider is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = 	Typhoeus.get(
						          "#{@endpoint}/api/v1/xenserver_services/net_list",
						          body: {
						            authenticate_token: @api_key,
						            email:              @email,
						            id:                 @name_or_id
						          } 
						        )

			    	data = JSON.parse data.response_body

			    	if data['success']
							Helpers::Table.table_callback_retrieve_data(data['netlist'], "List Xenserver Net")
			    	else
			    		Helpers::AlertMessage.show_error_message(data['message'])
			    	end
					end			
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# get json credential file
			# 
			def export
				@errors << "ID or Slug is required." if @name_or_id.blank?

				if valid?
					Helpers::ErrorException.rescue do
						data = Typhoeus.get(
		            "#{@endpoint}/api/v1/xenserver_services/export",
		            body: {
		              authenticate_token:   @api_key,
		              email:                @email,
		              id:                   @name_or_id
		            }
		          )

		        data = JSON.parse data.response_body rescue nil
		        user_provider = data["user_provider"]

						if user_provider.present?
		        	name_file = "xenserver_provider"
		        	Helpers::FileGenerator.create(name_file, user_provider)
		          Helpers::AlertMessage.show_success_message("Success export xenserver provider. Your file is saved on json file with name #{name_file}.json")
		        else
		        	Helpers::AlertMessage.show_error_message(data["message"])
		        end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# check valid json params
			# 
			def validate(file_path)
				@errors << "File directory is required." if file_path.blank?

				begin
					file    = File.open(file_path, "r")
					content = file.read
	        json    = JSON.parse(content)
	        encode  = Helpers::JsonBaseConvert.encode(json)
				rescue
					@errors << "Error on file. Please check again."
				end

				@errors << "Content of file is required." if content.blank?

				if valid?
					Helpers::ErrorException.rescue do

	          response = Typhoeus.get(
	            "#{@endpoint}/api/v1/openstack_services/validate",
	            body: {
	              authenticate_token: @api_key,
	              email:              @email,
	              content:            encode
	            }
	          )

	          @response = JSON.parse response.response_body

	          if @response["success"]
							Helpers::AlertMessage.show_success_message(@response["message"])
	          else
							Helpers::AlertMessage.show_error_message(@response["message"])
	          end
					end
				else
					Helpers::AlertMessage.show_error_message(error)
				end
			end

			# 
			# check no errors
			# 
			def valid?
				@errors.blank?
			end

			# 
			# call errors
			# 
			def error
				@errors.first
			end

		end	
	end
end